--===================================================
--									
--								GHI Position
--								GHI_Position.lua
--
--			Information about position of the user
--	
-- 						(c)2013 The Gryphonheart Team
--								All rights reserved
--===================================================

local class
function GHI_Position()
	if class then
		return class
	end
	class = GHClass("GHI_Position")

	--local variables
	local continent
	local currentContinent
	local x
	local y
	local zoneIndex
	local areaID
	local floorLevel;
	local dungeonLevel;
	local dungeonLevelDropDownShown;

	GHI_Event("GHP_LOADED",function()
		if GHP_FloorLevelDetermination then
			floorLevel = GHP_FloorLevelDetermination();
		end
	end);


	local Round = function(num,decimals)
		if decimals then
			return tonumber(string.format("%."..decimals.."f",num));
		end
		return num;
	end

	local ConvertMopToCata = function(x_mop,y_mop)
		local x_cata = 1.243338591 * x_mop - 2822.734638;
		local y_cata = 1.243339374 * y_mop - 409.9540594;
		return x_cata,y_cata;
	end

	local IsMopClient = function()
		local _, _, _, tocVersion = GetBuildInfo()
		return tocVersion >= 50000;
	end

	local ResetMap = function() --print("restting to id",areaID)
		SetMapToCurrentZone();
		if GetCurrentMapAreaID() == areaID then
			SetDungeonMapLevel(dungeonLevel);
		else
			SetMapByID(areaID)
		end
		if dungeonLevelDropDownShown then
			DropDownList1:Show();
		end
	end
	--Class functions
	class.GetCoor = function(unit,decimals)
		if not (unit) then unit = "player" end
		--if not(DO_ONE) then return 0,0,0; end
		--DO_ONE = nil;
		zoneIndex = GetCurrentMapZone()
		continent = GetCurrentMapContinent()

		areaID = GetCurrentMapAreaID()
		dungeonLevel = GetCurrentMapDungeonLevel();
		--print("id is",areaID)
		-- check if drop down is shown
		if DropDownList1:IsShown() and ({DropDownList1:GetPoint(1)})[2] == WorldMapLevelDropDownLeft then
			dungeonLevelDropDownShown = true;
		else
			dungeonLevelDropDownShown = false;
		end

		-- Move map
		SetMapToCurrentZone()
		currentContinent = GetCurrentMapContinent()
		if currentContinent < 0 or currentContinent > 6 or currentContinent == 5 then --6 is pandaria, 5 is maelstrom
			ResetMap()
			return 0.0, 0.0, 0
		end

		if currentContinent == 3 then
			SetMapZoom(3)
		else
			SetMapZoom(0)
		end

		local x, y = GetPlayerMapPosition(unit)
		local level;
		if floorLevel then
			level = floorLevel.GetCurrentFloorLevel();
		end

		if currentContinent == 3 then
			x = x * 2228.61382 -- scale for Outland
			y = y * 1485.74255
			ResetMap();
			return Round(x,decimals), Round(y,decimals), 2, level
		else
			x = x * 11698.9534 -- scale for Azeroth
			y = y * 7799.30229
			ResetMap();

			if IsMopClient() then
				x,y = ConvertMopToCata(x,y);
			end

			return Round(x,decimals), Round(y,decimals), 1, level
		end
	end

	class.GetPlayerPos = function(decimals)
		local x, y, world, level = class.GetCoor("player",decimals);
		return {
			x = x,
			y = y,
			world = world,
			level = level,
		};
	end

	class.IsPosWithinRange = function(position, range)
		GHCheck("GHI_Position.IsPosWithinRange", { "Table", "Number" }, { position, range })
		local playerPos = class.GetPlayerPos();
		position.world = position.world or position.continent;

		if not (playerPos.world == position.world) then
			return false;
		end
		if not(playerPos.level == position.level) then
			return false;
		end

		local xDiff = position.x - playerPos.x;
		local yDiff = position.y - playerPos.y;
		return math.abs(math.sqrt(xDiff * xDiff + yDiff * yDiff)) <= range
	end

	class.OnNextMoveCallback = function(range,func,_repeat)
		local pos = class.GetPlayerPos(3);
		GHI_Timer(function()
			if pos then
				if not(class.IsPosWithinRange(pos,range)) then
					func();
					if _repeat then
						pos = class.GetPlayerPos(3);
					else
						pos = nil;
					end
				end
			end
		end,1);
	end

	class.GetDirectionAndDistance = function(otherPos)
		local playerPos = class.GetPlayerPos();

		if not(otherPos.world == playerPos.world) then
			return 99999999
		end

		local dx = otherPos.x - playerPos.x;
		local dy = otherPos.y - playerPos.y;

		local dist = sqrt(dx^2+dy^2);

		local degreeAngle
		if dx >= 0 and dy >= 0 then
			degreeAngle = atan(dy/dx)+90;
		elseif dy >= 0 then
			degreeAngle = atan(-dx/dy)+180;
		elseif dx >= 0 then
			degreeAngle = atan(dx/-dy);
		else
			degreeAngle = atan(-dy/-dx)+270;
		end

		local directionNames = {"north","northeast","east","southeast","south","southwest","west","northwest","north"};
		local directionName = directionNames[math.floor((degreeAngle+22.5)/45)+1];

		local c = tostring(math.floor(dist)):len()-1;
		local roundDist = dist/(10^c);

		local distanceText = string.format("around %s%s meter%s",math.floor(roundDist+0.5),strrep("0",c),( roundDist >= 2 and "s") or "");

		return degreeAngle,directionName,dist,distanceText;
	end

	return class
end
